from fileformats.generic.file import File
from fileformats.medimage.nifti import Nifti1
from pathlib import Path
from pydra.engine import ShellCommandTask
from pydra.engine import specs

input_fields = [
    (
        "in_file",
        Nifti1,
        {
            "help_string": "input file to skull strip",
            "argstr": "{in_file}",
            "copyfile": False,
            "mandatory": True,
            "position": 0,
        },
    ),
    (
        "out_file",
        Path,
        {
            "help_string": "name of output skull stripped image",
            "argstr": "{out_file}",
            "position": 1,
            "output_file_template": '"brain_anat.nii"',
        },
    ),
    ("outline", bool, {"help_string": "create surface outline image", "argstr": "-o"}),
    ("mask", bool, {"help_string": "create binary mask image", "argstr": "-m"}),
    ("skull", bool, {"help_string": "create skull image", "argstr": "-s"}),
    (
        "no_output",
        bool,
        {"help_string": "Don't generate segmented output", "argstr": "-n"},
    ),
    (
        "frac",
        float,
        {"help_string": "fractional intensity threshold", "argstr": "-f {frac:.2}"},
    ),
    (
        "vertical_gradient",
        float,
        {
            "help_string": "vertical gradient in fractional intensity threshold (-1, 1)",
            "argstr": "-g {vertical_gradient:.2}",
        },
    ),
    ("radius", int, {"help_string": "head radius", "argstr": "-r {radius}"}),
    (
        "center",
        list,
        {"help_string": "center of gravity in voxels", "argstr": "-c {center}"},
    ),
    (
        "threshold",
        bool,
        {
            "help_string": "apply thresholding to segmented brain image and mask",
            "argstr": "-t",
        },
    ),
    (
        "mesh",
        bool,
        {"help_string": "generate a vtk mesh brain surface", "argstr": "-e"},
    ),
    (
        "robust",
        bool,
        {
            "help_string": "robust brain centre estimation (iterates BET several times)",
            "argstr": "-R",
            "xor": (
                "functional",
                "reduce_bias",
                "robust",
                "padding",
                "remove_eyes",
                "surfaces",
                "t2_guided",
            ),
        },
    ),
    (
        "padding",
        bool,
        {
            "help_string": "improve BET if FOV is very small in Z (by temporarily padding end slices)",
            "argstr": "-Z",
            "xor": (
                "functional",
                "reduce_bias",
                "robust",
                "padding",
                "remove_eyes",
                "surfaces",
                "t2_guided",
            ),
        },
    ),
    (
        "remove_eyes",
        bool,
        {
            "help_string": "eye & optic nerve cleanup (can be useful in SIENA)",
            "argstr": "-S",
            "xor": (
                "functional",
                "reduce_bias",
                "robust",
                "padding",
                "remove_eyes",
                "surfaces",
                "t2_guided",
            ),
        },
    ),
    (
        "surfaces",
        bool,
        {
            "help_string": "run bet2 and then betsurf to get additional skull and scalp surfaces (includes registrations)",
            "argstr": "-A",
            "xor": (
                "functional",
                "reduce_bias",
                "robust",
                "padding",
                "remove_eyes",
                "surfaces",
                "t2_guided",
            ),
        },
    ),
    (
        "t2_guided",
        File,
        {
            "help_string": "as with creating surfaces, when also feeding in non-brain-extracted T2 (includes registrations)",
            "argstr": "-A2 {t2_guided}",
            "xor": (
                "functional",
                "reduce_bias",
                "robust",
                "padding",
                "remove_eyes",
                "surfaces",
                "t2_guided",
            ),
        },
    ),
    (
        "functional",
        bool,
        {
            "help_string": "apply to 4D fMRI data",
            "argstr": "-F",
            "xor": (
                "functional",
                "reduce_bias",
                "robust",
                "padding",
                "remove_eyes",
                "surfaces",
                "t2_guided",
            ),
        },
    ),
    (
        "reduce_bias",
        bool,
        {
            "help_string": "bias field and neck cleanup",
            "argstr": "-B",
            "xor": (
                "functional",
                "reduce_bias",
                "robust",
                "padding",
                "remove_eyes",
                "surfaces",
                "t2_guided",
            ),
        },
    ),
]
BET_input_spec = specs.SpecInfo(
    name="Input", fields=input_fields, bases=(specs.ShellSpec,)
)

output_fields = [
    (
        "mask_file",
        File,
        {"help_string": "path/name of binary brain mask (if generated)"},
    ),
    ("outline_file", File, {"help_string": "path/name of outline file (if generated)"}),
    ("meshfile", File, {"help_string": "path/name of vtk mesh file (if generated)"}),
    (
        "inskull_mask_file",
        File,
        {"help_string": "path/name of inskull mask (if generated)"},
    ),
    (
        "inskull_mesh_file",
        File,
        {"help_string": "path/name of inskull mesh outline (if generated)"},
    ),
    (
        "outskull_mask_file",
        File,
        {"help_string": "path/name of outskull mask (if generated)"},
    ),
    (
        "outskull_mesh_file",
        File,
        {"help_string": "path/name of outskull mesh outline (if generated)"},
    ),
    (
        "outskin_mask_file",
        File,
        {"help_string": "path/name of outskin mask (if generated)"},
    ),
    (
        "outskin_mesh_file",
        File,
        {"help_string": "path/name of outskin mesh outline (if generated)"},
    ),
    (
        "skull_mask_file",
        File,
        {"help_string": "path/name of skull mask (if generated)"},
    ),
    ("skull_file", File, {"help_string": "path/name of skull file (if generated)"}),
]
BET_output_spec = specs.SpecInfo(
    name="Output", fields=output_fields, bases=(specs.ShellOutSpec,)
)


class BET(ShellCommandTask):
    """
    Examples
    -------

    >>> from fileformats.generic.file import File
    >>> from fileformats.medimage.nifti import Nifti1
    >>> from pydra.tasks.fsl.auto.bet import BET

    >>> task = BET()
    >>> task.inputs.in_file = Nifti1.mock("structural.nii")
    >>> task.inputs.out_file = "brain_anat.nii"
    >>> task.inputs.frac = 0.7
    >>> task.inputs.t2_guided = File.mock()
    >>> task.cmdline
    'bet structural.nii brain_anat.nii -f 0.70'


    """

    input_spec = BET_input_spec
    output_spec = BET_output_spec
    executable = "bet"
