from fileformats.generic.file import File
from fileformats.medimage.diffusion import Bval
from fileformats.medimage.diffusion import Bvec
from fileformats.medimage.nifti import Nifti1
from fileformats.text import TextFile
from pydra.engine import ShellCommandTask
from pydra.engine import specs
import typing as ty

input_fields = [
    (
        "in_file",
        Nifti1,
        {
            "help_string": "File containing all the images to estimate distortions for",
            "argstr": "--imain={in_file}",
            "mandatory": True,
        },
    ),
    (
        "in_mask",
        File,
        {
            "help_string": "Mask to indicate brain",
            "argstr": "--mask={in_mask}",
            "mandatory": True,
        },
    ),
    (
        "in_index",
        TextFile,
        {
            "help_string": "File containing indices for all volumes in --imain into --acqp and --topup",
            "argstr": "--index={in_index}",
            "mandatory": True,
        },
    ),
    (
        "in_acqp",
        File,
        {
            "help_string": "File containing acquisition parameters",
            "argstr": "--acqp={in_acqp}",
            "mandatory": True,
        },
    ),
    (
        "in_bvec",
        Bvec,
        {
            "help_string": "File containing the b-vectors for all volumes in --imain",
            "argstr": "--bvecs={in_bvec}",
            "mandatory": True,
        },
    ),
    (
        "in_bval",
        Bval,
        {
            "help_string": "File containing the b-values for all volumes in --imain",
            "argstr": "--bvals={in_bval}",
            "mandatory": True,
        },
    ),
    (
        "out_base",
        str,
        "eddy_corrected",
        {"help_string": "Basename for output image", "argstr": "--out={out_base}"},
    ),
    (
        "session",
        File,
        {
            "help_string": "File containing session indices for all volumes in --imain",
            "argstr": "--session={session}",
        },
    ),
    (
        "in_topup_fieldcoef",
        File,
        {
            "help_string": "Topup results file containing the field coefficients",
            "argstr": "--topup={in_topup_fieldcoef}",
            "requires": ["in_topup_movpar"],
        },
    ),
    (
        "in_topup_movpar",
        File,
        {
            "help_string": "Topup results file containing the movement parameters (movpar.txt)",
            "requires": ["in_topup_fieldcoef"],
        },
    ),
    (
        "field",
        File,
        {
            "help_string": "Non-topup derived fieldmap scaled in Hz",
            "argstr": "--field={field}",
        },
    ),
    (
        "field_mat",
        File,
        {
            "help_string": "Matrix specifying the relative positions of the fieldmap, --field, and the first volume of the input file, --imain",
            "argstr": "--field_mat={field_mat}",
        },
    ),
    (
        "flm",
        ty.Any,
        "quadratic",
        {"help_string": "First level EC model", "argstr": "--flm={flm}"},
    ),
    (
        "slm",
        ty.Any,
        "none",
        {"help_string": "Second level EC model", "argstr": "--slm={slm}"},
    ),
    (
        "fep",
        bool,
        {"help_string": "Fill empty planes in x- or y-directions", "argstr": "--fep"},
    ),
    (
        "initrand",
        bool,
        {
            "help_string": "Resets rand for when selecting voxels",
            "argstr": "--initrand",
        },
    ),
    (
        "interp",
        ty.Any,
        "spline",
        {
            "help_string": "Interpolation model for estimation step",
            "argstr": "--interp={interp}",
        },
    ),
    (
        "nvoxhp",
        int,
        1000,
        {
            "help_string": "# of voxels used to estimate the hyperparameters",
            "argstr": "--nvoxhp={nvoxhp}",
        },
    ),
    (
        "fudge_factor",
        float,
        10.0,
        {
            "help_string": "Fudge factor for hyperparameter error variance",
            "argstr": "--ff={fudge_factor}",
        },
    ),
    (
        "dont_sep_offs_move",
        bool,
        {
            "help_string": "Do NOT attempt to separate field offset from subject movement",
            "argstr": "--dont_sep_offs_move",
        },
    ),
    (
        "dont_peas",
        bool,
        {
            "help_string": "Do NOT perform a post-eddy alignment of shells",
            "argstr": "--dont_peas",
        },
    ),
    (
        "fwhm",
        float,
        {
            "help_string": "FWHM for conditioning filter when estimating the parameters",
            "argstr": "--fwhm={fwhm}",
        },
    ),
    (
        "niter",
        int,
        5,
        {"help_string": "Number of iterations", "argstr": "--niter={niter}"},
    ),
    (
        "method",
        ty.Any,
        "jac",
        {
            "help_string": "Final resampling method (jacobian/least squares)",
            "argstr": "--resamp={method}",
        },
    ),
    (
        "repol",
        bool,
        {"help_string": "Detect and replace outlier slices", "argstr": "--repol"},
    ),
    (
        "outlier_nstd",
        int,
        {
            "help_string": "Number of std off to qualify as outlier",
            "argstr": "--ol_nstd",
            "requires": ["repol"],
        },
    ),
    (
        "outlier_nvox",
        int,
        {
            "help_string": "Min # of voxels in a slice for inclusion in outlier detection",
            "argstr": "--ol_nvox",
            "requires": ["repol"],
        },
    ),
    (
        "outlier_type",
        ty.Any,
        {
            "help_string": "Type of outliers, slicewise (sw), groupwise (gw) or both (both)",
            "argstr": "--ol_type",
            "requires": ["repol"],
        },
    ),
    (
        "outlier_pos",
        bool,
        {
            "help_string": "Consider both positive and negative outliers if set",
            "argstr": "--ol_pos",
            "requires": ["repol"],
        },
    ),
    (
        "outlier_sqr",
        bool,
        {
            "help_string": "Consider outliers among sums-of-squared differences if set",
            "argstr": "--ol_sqr",
            "requires": ["repol"],
        },
    ),
    (
        "multiband_factor",
        int,
        {"help_string": "Multi-band factor", "argstr": "--mb={multiband_factor}"},
    ),
    (
        "multiband_offset",
        ty.Any,
        {
            "help_string": "Multi-band offset (-1 if bottom slice removed, 1 if top slice removed",
            "argstr": "--mb_offs={multiband_offset}",
            "requires": ["multiband_factor"],
        },
    ),
    (
        "mporder",
        int,
        {
            "help_string": "Order of slice-to-vol movement model",
            "argstr": "--mporder={mporder}",
            "requires": ["use_cuda"],
        },
    ),
    (
        "slice2vol_niter",
        int,
        {
            "help_string": "Number of iterations for slice-to-vol",
            "argstr": "--s2v_niter={slice2vol_niter}",
            "requires": ["mporder"],
        },
    ),
    (
        "slice2vol_lambda",
        int,
        {
            "help_string": "Regularisation weight for slice-to-vol movement (reasonable range 1-10)",
            "argstr": "--s2v_lambda={slice2vol_lambda}",
            "requires": ["mporder"],
        },
    ),
    (
        "slice2vol_interp",
        ty.Any,
        {
            "help_string": "Slice-to-vol interpolation model for estimation step",
            "argstr": "--s2v_interp={slice2vol_interp}",
            "requires": ["mporder"],
        },
    ),
    (
        "slice_order",
        TextFile,
        {
            "help_string": "Name of text file completely specifying slice/group acquisition",
            "argstr": "--slspec={slice_order}",
            "requires": ["mporder"],
            "xor": ["json"],
        },
    ),
    (
        "json",
        File,
        {
            "help_string": "Name of .json text file with information about slice timing",
            "argstr": "--json={json}",
            "requires": ["mporder"],
            "xor": ["slice_order"],
        },
    ),
    (
        "estimate_move_by_susceptibility",
        bool,
        {
            "help_string": "Estimate how susceptibility field changes with subject movement",
            "argstr": "--estimate_move_by_susceptibility",
        },
    ),
    (
        "mbs_niter",
        int,
        {
            "help_string": "Number of iterations for MBS estimation",
            "argstr": "--mbs_niter={mbs_niter}",
            "requires": ["estimate_move_by_susceptibility"],
        },
    ),
    (
        "mbs_lambda",
        int,
        {
            "help_string": "Weighting of regularisation for MBS estimation",
            "argstr": "--mbs_lambda={mbs_lambda}",
            "requires": ["estimate_move_by_susceptibility"],
        },
    ),
    (
        "mbs_ksp",
        int,
        {
            "help_string": "Knot-spacing for MBS field estimation",
            "argstr": "--mbs_ksp={mbs_ksp}mm",
            "requires": ["estimate_move_by_susceptibility"],
        },
    ),
    ("num_threads", int, 1, {"help_string": "Number of openmp threads to use"}),
    (
        "is_shelled",
        bool,
        {
            "help_string": "Override internal check to ensure that date are acquired on a set of b-value shells",
            "argstr": "--data_is_shelled",
        },
    ),
    ("use_cuda", bool, {"help_string": "Run eddy using cuda gpu"}),
    ("cnr_maps", bool, {"help_string": "Output CNR-Maps", "argstr": "--cnr_maps"}),
    ("residuals", bool, {"help_string": "Output Residuals", "argstr": "--residuals"}),
]
Eddy_input_spec = specs.SpecInfo(
    name="Input", fields=input_fields, bases=(specs.ShellSpec,)
)

output_fields = [
    (
        "out_corrected",
        File,
        {"help_string": "4D image file containing all the corrected volumes"},
    ),
    (
        "out_parameter",
        File,
        {
            "help_string": "Text file with parameters defining the field and movement for each scan"
        },
    ),
    (
        "out_rotated_bvecs",
        File,
        {"help_string": "File containing rotated b-values for all volumes"},
    ),
    (
        "out_movement_rms",
        File,
        {"help_string": "Summary of the 'total movement' in each volume"},
    ),
    (
        "out_restricted_movement_rms",
        File,
        {
            "help_string": "Summary of the 'total movement' in each volume disregarding translation in the PE direction"
        },
    ),
    (
        "out_shell_alignment_parameters",
        File,
        {
            "help_string": "Text file containing rigid body movement parameters between the different shells as estimated by a post-hoc mutual information based registration"
        },
    ),
    (
        "out_shell_pe_translation_parameters",
        File,
        {
            "help_string": "Text file containing translation along the PE-direction between the different shells as estimated by a post-hoc mutual information based registration"
        },
    ),
    (
        "out_outlier_map",
        File,
        {
            "help_string": 'Matrix where rows represent volumes and columns represent slices. "0" indicates that scan-slice is not an outlier and "1" indicates that it is'
        },
    ),
    (
        "out_outlier_n_stdev_map",
        File,
        {
            "help_string": "Matrix where rows represent volumes and columns represent slices. Values indicate number of standard deviations off the mean difference between observation and prediction is"
        },
    ),
    (
        "out_outlier_n_sqr_stdev_map",
        File,
        {
            "help_string": "Matrix where rows represent volumes and columns represent slices. Values indicate number of standard deivations off the square root of the mean squared difference between observation and prediction is"
        },
    ),
    (
        "out_outlier_report",
        File,
        {
            "help_string": "Text file with a plain language report on what outlier slices eddy has found"
        },
    ),
    (
        "out_outlier_free",
        File,
        {
            "help_string": "4D image file not corrected for susceptibility or eddy-current distortions or subject movement but with outlier slices replaced"
        },
    ),
    (
        "out_movement_over_time",
        File,
        {
            "help_string": "Text file containing translations (mm) and rotations (radians) for each excitation"
        },
    ),
    ("out_cnr_maps", File, {"help_string": "path/name of file with the cnr_maps"}),
    ("out_residuals", File, {"help_string": "path/name of file with the residuals"}),
]
Eddy_output_spec = specs.SpecInfo(
    name="Output", fields=output_fields, bases=(specs.ShellOutSpec,)
)


class Eddy(ShellCommandTask):
    """
    Examples
    -------

    >>> from fileformats.generic.file import File
    >>> from fileformats.medimage.diffusion import Bval
    >>> from fileformats.medimage.diffusion import Bvec
    >>> from fileformats.medimage.nifti import Nifti1
    >>> from fileformats.text import TextFile
    >>> from pydra.tasks.fsl.auto.eddy import Eddy

    >>> task = Eddy()
    >>> task.inputs.in_file = Nifti1.mock("epi.nii")
    >>> task.inputs.in_mask = File.mock()
    >>> task.inputs.in_index = TextFile.mock("epi_index.txt")
    >>> task.inputs.in_acqp = File.mock()
    >>> task.inputs.in_bvec = Bvec.mock()
    >>> task.inputs.in_bval = Bval.mock()
    >>> task.inputs.session = File.mock()
    >>> task.inputs.in_topup_fieldcoef = File.mock()
    >>> task.inputs.in_topup_movpar = File.mock()
    >>> task.inputs.field = File.mock()
    >>> task.inputs.field_mat = File.mock()
    >>> task.inputs.slice_order = TextFile.mock()
    >>> task.inputs.json = File.mock()
    >>> task.cmdline
    'eddy_openmp --flm=quadratic --ff=10.0 --acqp=epi_acqp.txt --bvals=bvals.scheme --bvecs=bvecs.scheme --imain=epi.nii --index=epi_index.txt --mask=epi_mask.nii --interp=spline --resamp=jac --niter=5 --nvoxhp=1000 --out=.../eddy_corrected --slm=none'


    >>> task = Eddy()
    >>> task.inputs.in_file = Nifti1.mock()
    >>> task.inputs.in_mask = File.mock()
    >>> task.inputs.in_index = TextFile.mock()
    >>> task.inputs.in_acqp = File.mock()
    >>> task.inputs.in_bvec = Bvec.mock()
    >>> task.inputs.in_bval = Bval.mock()
    >>> task.inputs.session = File.mock()
    >>> task.inputs.in_topup_fieldcoef = File.mock()
    >>> task.inputs.in_topup_movpar = File.mock()
    >>> task.inputs.field = File.mock()
    >>> task.inputs.field_mat = File.mock()
    >>> task.inputs.slice_order = TextFile.mock()
    >>> task.inputs.json = File.mock()
    >>> task.inputs.use_cuda = True
    >>> task.cmdline
    'eddy_cuda --flm=quadratic --ff=10.0 --acqp=epi_acqp.txt --bvals=bvals.scheme --bvecs=bvecs.scheme --imain=epi.nii --index=epi_index.txt --mask=epi_mask.nii --interp=spline --resamp=jac --niter=5 --nvoxhp=1000 --out=.../eddy_corrected --slm=none'


    >>> task = Eddy()
    >>> task.inputs.in_file = Nifti1.mock()
    >>> task.inputs.in_mask = File.mock()
    >>> task.inputs.in_index = TextFile.mock()
    >>> task.inputs.in_acqp = File.mock()
    >>> task.inputs.in_bvec = Bvec.mock()
    >>> task.inputs.in_bval = Bval.mock()
    >>> task.inputs.session = File.mock()
    >>> task.inputs.in_topup_fieldcoef = File.mock()
    >>> task.inputs.in_topup_movpar = File.mock()
    >>> task.inputs.field = File.mock()
    >>> task.inputs.field_mat = File.mock()
    >>> task.inputs.mporder = 6
    >>> task.inputs.slice2vol_niter = 5
    >>> task.inputs.slice2vol_lambda = 1
    >>> task.inputs.slice2vol_interp = "trilinear"
    >>> task.inputs.slice_order = TextFile.mock("epi_slspec.txt")
    >>> task.inputs.json = File.mock()
    >>> task.cmdline
    'eddy_cuda --flm=quadratic --ff=10.0 --acqp=epi_acqp.txt --bvals=bvals.scheme --bvecs=bvecs.scheme --imain=epi.nii --index=epi_index.txt --mask=epi_mask.nii --interp=spline --resamp=jac --mporder=6 --niter=5 --nvoxhp=1000 --out=.../eddy_corrected --s2v_interp=trilinear --s2v_lambda=1 --s2v_niter=5 --slspec=epi_slspec.txt --slm=none'


    """

    input_spec = Eddy_input_spec
    output_spec = Eddy_output_spec
    executable = "eddy_openmp"
